use std::collections::HashMap;

use petgraph::{
    adj::EdgeIndex, data::Build, graph::DiGraph, graphmap::DiGraphMap, prelude::NodeIndex,
};

pub mod length;
pub mod nodes;

use crate::types::nodes::primitives::color_matrix::{ColorMatrix, ColorMatrixType};

use self::{
    length::{Coordinate, Length},
    nodes::Node,
};

#[derive(Debug)]
pub struct Filter<'a> {
    pub graph: DiGraph<Node, Edge<'a>>,
    source_graphic_idx: NodeIndex,
}

impl Filter<'_> {
    pub fn new() -> Self {
        let mut graph = DiGraph::new();
        let source_graphic_idx = graph.add_node(Node::StdInput(
            nodes::standard_input::StandardInput::SourceGraphic,
        ));

        Self {
            graph,
            source_graphic_idx,
        }
    }

    pub fn add_node(&mut self, node: Node) -> NodeIndex {
        self.graph.add_node(node)
    }

    pub fn source_graphic(&self) -> NodeIndex {
        self.source_graphic_idx
    }
}

impl Default for Filter<'_> {
    fn default() -> Self {
        Self::new()
    }
}

#[derive(Debug, Clone)]
pub struct Edge<'a> {
    edge_type: EdgeType<'a>,
    /// the index of the `in` attribute on the target element
    /// if None, just `in`
    in_idx: Option<u8>,
}

impl Edge<'_> {
    pub fn new() -> Self {
        Self {
            edge_type: EdgeType::Unnamed,
            in_idx: None,
        }
    }

    pub fn in_idx(idx: u8) -> Self {
        Self {
            edge_type: EdgeType::Unnamed,
            in_idx: Some(idx),
        }
    }
}

#[derive(Debug, Clone)]
pub enum EdgeType<'a> {
    Named(&'a str),
    /// For standard inputs such as SourceGraphic etc., which we'll just be representing as nodes for simplicity
    Unnamed,
}
